<?php

 /* 404cache.php -- Makes pmwiki into a 404-caching server
  * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

if (!defined('PmWiki')) exit();

define('FOUR_OH_FOUR_CACHE_VERSION', 0.1);

$PubDirParts = parse_url($PubDirUrl);
$PubDirPath = $PubDirParts['path'];

SDV($CacheDir, "pub/page");
SDV($CachePath, "$PubDirPath/page");

$CachedBrowseFunction = $HandleActions['browse'];
if (!$CachedBrowseFunction) $CachedBrowseFunction = 'HandleBrowse';
$HandleActions['browse'] = 'FourOhFourBrowse';

if ($_SERVER['REDIRECT_STATUS'] == 404) {
    $pagename = CacheUrlToPagename($_SERVER['REDIRECT_URL']);
}

# So that browses will find our cache pages
$PageUrlFmt = $CachePath . '/$Group/$Title_';

if ($action == 'edit') {
    # XXX: default is $PageUrl, which is wrong
    $PageEditFmt[0] = preg_replace("/action=['\"].*?['\"]/",
				   "action='\$ScriptUrl?pagename=\$Group/\$Title&action=post'",
				   $PageEditFmt[0]);
} else if (strpos($action, 'post') === 0) {
    # Invalidate cache on post
    InvalidateCache($pagename);
}

if (!function_exists('file_put_contents')) {
    # Not exactly correct, but close enough
    function file_put_contents($filename, $contents) {
	$handle = fopen($filename, 'w');
	fwrite($handle, $contents);
	fclose($handle);
    }
}

# We use output buffering to grab all the output and store it in the cache
# file before dumping it to the browser

function FourOhFourBrowse($pagename) {
    global $CachedBrowseFunction;
    global $CacheDir;
    EnsureCacheDir($CacheDir);
    ob_start();
    $CachedBrowseFunction($pagename);
    $contents = ob_get_contents();
    WriteCacheFile($pagename, $contents);
    ob_end_flush();
}

function WriteCacheFile($pagename, $contents) {
    global $CacheDir;
    $gt = GroupAndTitle($pagename);
    $group_dir = $CacheDir . '/' . $gt[0];
    if (!file_exists($group_dir)) {
	mkdir($group_dir);
    }
    file_put_contents($group_dir . '/' . $gt[1] . '.html', $contents);
}

function InvalidateCache($pagename) {
    global $CacheDir;
    EnsureCacheDir($CacheDir);
    DeleteAllFiles($CacheDir);
}

function DeleteAllFiles($dir) {
    $dh = opendir($dir);
    while (($filename = readdir($dh)) !== false) {
        if ($filename != "." && $filename != "..") {
	    $fullname = $dir . '/' . $filename;
	    if (is_dir($fullname)) {
		DeleteAllFiles($fullname);
		rmdir($fullname);
	    } else {
		unlink($fullname);
	    }
	}
    }
    closedir($dh);
}

function CacheUrlToPageName($url) {
    global $CachePath;
    $parts = parse_url($url);
    return substr($parts['path'], strlen($CachePath) + 1);
}

function PageNameToCacheFileName($pagename) {
    global $CacheDir;
    $gt = GroupAndTitle($pagename);
    return $CacheDir . '/' . $gt[0] . '/' . $gt[1] . '.html';
}

function GroupAndTitle($pagename) {
    global $GroupNamePattern, $PageTitlePattern;
    if (preg_match("/^($GroupNamePattern)[\\/.]($PageTitlePattern)\$/",
		    $pagename, $match)) {
	return array($match[1], $match[2]);
    } else {
	return array('', '');
    }
}

function EnsureCacheDir($cachedir) {
    
    if (!file_exists($cachedir)) {
	Abort("Cache directory '$cachedir' does not exist!");
    }
    
    if (!is_dir($cachedir)) {
	Abort("Cache directory '$cachedir' does not exist!");
    }
    
    if (!is_writeable($cachedir)) {
	Abort("Cache directory '$cachedir' is not writeable!");
    }
}

?>
